<?php

namespace Chemical\Subscriber\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Chemical\Subscriber\Http\Requests\SubscriberRequest;
use Chemical\Subscriber\Interfaces\SubscriberRepositoryInterface;
use Chemical\Subscriber\Models\Subscriber;

/**
 * Resource controller class for subscriber.
 */
class SubscriberResourceController extends BaseController
{

    /**
     * Initialize subscriber resource controller.
     *
     * @param type SubscriberRepositoryInterface $subscriber
     *
     * @return null
     */
    public function __construct(SubscriberRepositoryInterface $subscriber)
    {
        parent::__construct();
        $this->repository = $subscriber;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Chemical\Subscriber\Repositories\Criteria\SubscriberResourceCriteria::class);
    }

    /**
     * Display a list of subscriber.
     *
     * @return Response
     */
    public function index(SubscriberRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Chemical\Subscriber\Repositories\Presenter\SubscriberPresenter::class)
                ->$function();
        }

        $subscribers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('subscriber::subscriber.names'))
            ->view('subscriber::subscriber.index', true)
            ->data(compact('subscribers', 'view'))
            ->output();
    }

    /**
     * Display subscriber.
     *
     * @param Request $request
     * @param Model   $subscriber
     *
     * @return Response
     */
    public function show(SubscriberRequest $request, Subscriber $subscriber)
    {

        if ($subscriber->exists) {
            $view = 'subscriber::subscriber.show';
        } else {
            $view = 'subscriber::subscriber.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('subscriber::subscriber.name'))
            ->data(compact('subscriber'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subscriber.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubscriberRequest $request)
    {

        $subscriber = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('subscriber::subscriber.name')) 
            ->view('subscriber::subscriber.create', true) 
            ->data(compact('subscriber'))
            ->output();
    }

    /**
     * Create new subscriber.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubscriberRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subscriber                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('subscriber::subscriber.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subscriber/subscriber/' . $subscriber->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/subscriber/subscriber'))
                ->redirect();
        }

    }

    /**
     * Show subscriber for editing.
     *
     * @param Request $request
     * @param Model   $subscriber
     *
     * @return Response
     */
    public function edit(SubscriberRequest $request, Subscriber $subscriber)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('subscriber::subscriber.name'))
            ->view('subscriber::subscriber.edit', true)
            ->data(compact('subscriber'))
            ->output();
    }

    /**
     * Update the subscriber.
     *
     * @param Request $request
     * @param Model   $subscriber
     *
     * @return Response
     */
    public function update(SubscriberRequest $request, Subscriber $subscriber)
    {
        try {
            $attributes = $request->all();

            $subscriber->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('subscriber::subscriber.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subscriber/subscriber/' . $subscriber->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subscriber/subscriber/' . $subscriber->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subscriber.
     *
     * @param Model   $subscriber
     *
     * @return Response
     */
    public function destroy(SubscriberRequest $request, Subscriber $subscriber)
    {
        try {

            $subscriber->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subscriber::subscriber.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('subscriber/subscriber/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subscriber/subscriber/' . $subscriber->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subscriber.
     *
     * @param Model   $subscriber
     *
     * @return Response
     */
    public function delete(SubscriberRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subscriber::subscriber.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('subscriber/subscriber'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subscriber/subscriber'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subscribers.
     *
     * @param Model   $subscriber
     *
     * @return Response
     */
    public function restore(SubscriberRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('subscriber::subscriber.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/subscriber/subscriber'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subscriber/subscriber/'))
                ->redirect();
        }

    }

}
